@@ -51,6 +51,7 @@ class AutoTrader():
         self.home_dir       = None
         self.validation_file = None
         self.plot_validation_balance = True
+        self.include_broker = False
     
     def run(self):
         if self.show_help is not None:
@@ -117,8 +118,12 @@ class AutoTrader():
         strategy            = getattr(module, strat_name)
         
         if self.backtest is True:
-            utils           = importlib.import_module('autotrader.brokers.virtual.utils')
-            broker          = Broker(broker_config)
+            # TODO: generalise broker used 
+            # Could interanlly specify broker based on feed. Oanda feed -> oanda broker,
+            # Yahoo feed -> virtual broker. Doing this will generalise broker utils import
+            utils_module    = importlib.import_module('autotrader.brokers.virtual.utils')
+            utils           = utils_module.Utils()
+            broker          = Broker(broker_config, utils)
             
             from_date       = datetime.strptime(config['BACKTESTING']['FROM']+'+0000', '%d/%m/%Y%z')
             to_date         = datetime.strptime(config['BACKTESTING']['TO']+'+0000', '%d/%m/%Y%z')
@@ -152,8 +157,9 @@ class AutoTrader():
             
         else:
             # TODO generalise per broker used
-            utils           = importlib.import_module('autotrader.brokers.oanda.utils') # FOR OANDA ONLY
-            broker          = Oanda.Oanda(broker_config)
+            utils_module    = importlib.import_module('autotrader.brokers.oanda.utils') # FOR OANDA ONLY
+            utils           = utils_module.Utils()
+            broker          = Oanda.Oanda(broker_config, utils)
         
         if int(self.notify) > 0:
             
@@ -208,6 +214,7 @@ class AutoTrader():
                 starting_balance    = broker.get_balance()
                 NAV     = []
                 balance = []
+                margin  = []
                 
                 if self.data_file is not None:
                     custom_data_file        = self.data_file
@@ -345,6 +352,10 @@ class AutoTrader():
             # Instantiate strategy for current instrument
             my_strat    = strategy(params, data, instrument)
             
+            if self.include_broker:
+                my_strat.broker = broker
+                my_strat.broker_utils = utils
+            
             if int(self.verbosity) > 0:
                 print("\nAnalysing {}/{}".format(instrument[:3], instrument[-3:]),
                       "on {} timeframe using {}.".format(interval,
@@ -364,7 +375,6 @@ class AutoTrader():
                 candle              = data.iloc[i]
                 
                 # Run strategy to get signals
-                # TODO - move this into order_processing method
                 signal_dict = my_strat.generate_signal(i, open_positions)
                 if 0 not in signal_dict:
                     # Single order signal, nest in dictionary to allow iteration
@@ -373,7 +383,7 @@ class AutoTrader():
                     
                 # Begin iteration over signal_dict to extract each order
                 for order in signal_dict:
-                    order_signal_dict = signal_dict[order]
+                    order_signal_dict = signal_dict[order].copy()
                 
                     signal      = order_signal_dict["direction"]
                     
@@ -410,15 +420,21 @@ class AutoTrader():
                             working_price = order_price
                         
                         # Calculate exit levels
-                        stop_price = order_signal_dict['stop_loss'] if 'stop_loss' in order_signal_dict else None
+                        pip_value   = utils.get_pip_ratio(instrument)
                         stop_distance = order_signal_dict['stop_distance'] if 'stop_distance' in order_signal_dict else None
                         stop_type = order_signal_dict['stop_type'] if 'stop_type' in order_signal_dict else None
                         
+                        if 'stop_loss' not in order_signal_dict and \
+                            'stop_distance' in order_signal_dict and \
+                            order_signal_dict['stop_distance'] is not None:
+                            stop_price = working_price - np.sign(signal)*stop_distance*pip_value
+                        else:
+                            stop_price = order_signal_dict['stop_loss'] if 'stop_loss' in order_signal_dict else None
+                        
                         if 'take_profit' not in order_signal_dict and \
                             'take_distance' in order_signal_dict and \
                             order_signal_dict['take_distance'] is not None:
                             # Take profit distance specified
-                            pip_value   = utils.get_pip_ratio(instrument)
                             take_profit = working_price + np.sign(signal)*order_signal_dict['take_distance']*pip_value
                         else:
                             # Take profit price specified, or no take profit specified at all
@@ -496,6 +512,7 @@ class AutoTrader():
                     broker.update_positions(candle)
                     NAV.append(broker.NAV)
                     balance.append(broker.portfolio_balance)
+                    margin.append(broker.margin_available)
                     
                     
             # End data iteration 
@@ -523,6 +540,7 @@ class AutoTrader():
                         backtest_dict = {}
                         backtest_dict['data']           = data
                         backtest_dict['NAV']            = NAV
+                        backtest_dict['margin']         = margin
                         backtest_dict['trade_summary']  = trade_summary
                         backtest_dict['indicators']     = my_strat.indicators if hasattr(my_strat, 'indicators') else None
                         backtest_dict['pair']           = instrument
@@ -588,6 +606,7 @@ class AutoTrader():
                 # Cancelled orders (insufficient margin)
                 cancelled_orders    = broker.cancelled_orders
                 no_cancelled        = len(cancelled_orders)
+                no_open             = len(broker.open_positions)
                 
                 # Long trades
                 long_trades     = trade_summary[trade_summary.Size > 0]
@@ -630,6 +649,9 @@ class AutoTrader():
                     backtest_results['short_trades']['max_short_loss']  = max_short_loss
                     backtest_results['short_trades']['short_wr']        = short_wr
                 
+                # Save results
+                self.backtest_results = backtest_results
+                
             
             ''' -------------------------------------------------------------- '''
             '''                     Print output to console                    '''
@@ -662,8 +684,13 @@ class AutoTrader():
                     
                 else:
                     print("No trades taken.")
-                    
-                print("Cancelled orders:        {}".format(no_cancelled))
+                
+                
+                if no_open > 0:
+                    print("Orders still open:       {}".format(no_open))
+                if no_cancelled > 0:
+                    print("Cancelled orders:        {}".format(no_cancelled))
+                
                 
                 # Long trades
                 print("\n         Summary of long trades")
@@ -754,7 +781,7 @@ class AutoTrader():
         
         if self.backtest is True:
             return trade_summary
-    
+
 
 
     def read_yaml(self, file_path):
